#ifndef _VECTOR3D_H_
#define _VECTOR3D_H_

#include <math.h>
#include <iostream>

using namespace std;

class Vector3D
{
	friend inline ostream& operator << ( ostream& s, const Vector3D& V );

public:

	Vector3D();
	Vector3D( float x, float y, float z );

	inline void Set( float x, float y, float z ) { m_x = x; m_y = y; m_z = z; }

	Vector3D& operator += ( const Vector3D& V );
	Vector3D& operator -= ( const Vector3D& V );

	Vector3D operator + ( const Vector3D& V ) const;
	Vector3D operator - ( const Vector3D& V ) const;

	float operator * ( const Vector3D& V ) const;
	Vector3D operator ^ ( const Vector3D& V ) const;

	Vector3D operator * ( float S ) const;
	Vector3D operator / ( float S ) const;

	Vector3D& operator *= ( float S );
	Vector3D& operator /= ( float S );

	inline float x() const { return m_x; }
	inline float y() const { return m_y; }
	inline float z() const { return m_z; }

	inline float Len() const { return sqrt( m_x*m_x + m_y*m_y + m_z*m_z ); }
	inline float SLen() const { return m_x*m_x + m_y*m_y + m_z*m_z; }
	inline float MLen() const { return fabs( m_x ) + fabs( m_y ) + fabs( m_z ); }

	inline void Normalize() { float L = Len(); if( L > 0 ) ( *this ) /= Len(); }

	inline void Reverse() { m_x = -m_x; m_y = -m_y; m_z = -m_z; }

private:

	float m_x;
	float m_y;
	float m_z;


};


inline ostream& operator << ( ostream& s, const Vector3D& V )
{
	s << "(" << V.m_x << "," << V.m_y << "," << V.m_z << ")";
	return s;
}

inline Vector3D operator * ( float S, Vector3D V )
{
	return V*S;
}


#endif //_VECTOR3D_H_
